Package: BeeBDC
Title: Occurrence Data Cleaning
Version: 1.3.4
Authors@R: c(
    person("James B.", "Dorey", , "jbdorey@me.com", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0003-2721-3842")),
    person("Robert L.", "O'Reilly", , "robert.oreilly@flinders.edu.au", role = "aut",
           comment = c(ORCID = "0000-0001-5291-7396")),
    person("Silas", "Bossert", , "silas.bossert@wsu.edu", role = "aut",
           comment = c(ORCID = "0000-0002-3620-5468")),
    person("Erica E.", "Fischer", , "fischeer@mail.gvsu.edu", role = "aut",
           comment = c(ORCID = "0000-0002-8202-158X"))
  )
Description: Flags and checks occurrence data that are in Darwin Core
    format.  The package includes generic functions and data as well as
    some that are specific to bees. This package is meant to build upon
    and be complimentary to other excellent occurrence cleaning packages,
    including 'bdc' and 'CoordinateCleaner'.  This package uses datasets
    from several sources and particularly from the Discover Life Website,
    created by Ascher and Pickering (2020).  For further information,
    please see the original publication and package website.  Publication
    - Dorey et al. (2023) <doi:10.1101/2023.06.30.547152> and package
    website - Dorey et al. (2023) <https://github.com/jbdorey/BeeBDC>.
License: GPL (>= 3)
URL: https://jbdorey.github.io/BeeBDC/
        https://github.com/jbdorey/BeeBDC
BugReports: https://github.com/jbdorey/BeeBDC/issues
Depends: R (>= 4.1.0)
Imports: circlize, CoordinateCleaner, cowplot, dplyr, forcats, ggplot2,
        ggspatial, here, igraph, lubridate, mgsub, openxlsx, paletteer,
        readr, rnaturalearth, sf, stringr, tidyselect
Suggests: bdc, BiocManager, classInt, ComplexHeatmap, countrycode,
        curl, devtools, emld, formatR, galah, hexbin, htmltools,
        htmlwidgets, httr, iNEXT, janitor, kableExtra, knitr, leaflet,
        magrittr, mosaic, pkgdown, plotly, prettydoc, purrr, R.utils,
        renv, rgnparser, rlang, rmarkdown, rmdformats,
        rnaturalearthdata, rvest, SpadeR, taxadb, terra, testthat,
        tibble, tidyr, utils, xml2
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-gb
LazyData: TRUE
LazyDataCompression: xz
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-22 05:05:20 UTC; jamesdorey
Author: James B. Dorey [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0003-2721-3842>),
  Robert L. O'Reilly [aut] (ORCID:
    <https://orcid.org/0000-0001-5291-7396>),
  Silas Bossert [aut] (ORCID: <https://orcid.org/0000-0002-3620-5468>),
  Erica E. Fischer [aut] (ORCID: <https://orcid.org/0000-0002-8202-158X>)
Maintainer: James B. Dorey <jbdorey@me.com>
Repository: CRAN
Date/Publication: 2026-05-22 06:52:05 UTC
Built: R 4.6.0; ; 2026-05-22 12:04:33 UTC; unix
