# Generated by roxygen2: do not edit by hand

export("%>%")
export(BeeBDCQuery)
export(ChaoWrapper)
export(ColTypeR)
export(GBIFissues)
export(PaigeIntegrater)
export(USGS_formatter)
export(atlasDownloader)
export(beesChecklist)
export(beesTaxonomy)
export(chordDiagramR)
export(continentOutlieRs)
export(coordUncerFlagR)
export(countryHarmoniseR)
export(countryNameCleanR)
export(countryOutlieRs)
export(dataProvTables)
export(dataSaver)
export(dateFindR)
export(diagonAlley)
export(dirMaker)
export(dupePlotR)
export(dupeSummary)
export(fileFinder)
export(flagAbsent)
export(flagLicense)
export(flagRecorder)
export(flagSummaryTable)
export(formattedCombiner)
export(ggRichnessWrapper)
export(harmoniseR)
export(iNEXTwrapper)
export(idMatchR)
export(importOccurrences)
export(interactiveMapR)
export(jbd_CfC_chunker)
export(jbd_Ctrans_chunker)
export(jbd_coordCountryInconsistent)
export(jbd_coordinates_precision)
export(jbd_coordinates_transposed)
export(jbd_create_figures)
export(manualOutlierFindeR)
export(plotFlagSummary)
export(readr_BeeBDC)
export(repoFinder)
export(repoMerge)
export(richnessEstimateR)
export(richnessPrepR)
export(summaryFun)
export(summaryMaps)
export(taxadbToBeeBDC)
importFrom(CoordinateCleaner,cc_val)
importFrom(CoordinateCleaner,clean_coordinates)
importFrom(circlize,chordDiagram)
importFrom(circlize,circos.clear)
importFrom(circlize,circos.par)
importFrom(circlize,circos.trackPlotRegion)
importFrom(circlize,get.cell.meta.data)
importFrom(circlize,highlight.sector)
importFrom(circlize,mm_h)
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,as_tibble)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,contains)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,if_else)
importFrom(dplyr,left_join)
importFrom(dplyr,lst)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,n_groups)
importFrom(dplyr,pull)
importFrom(dplyr,relocate)
importFrom(dplyr,rename)
importFrom(dplyr,row_number)
importFrom(dplyr,select)
importFrom(dplyr,tibble)
importFrom(dplyr,where)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggspatial,north_arrow_fancy_orienteering)
importFrom(grDevices,gray)
importFrom(grid,unit)
importFrom(here,here)
importFrom(here,i_am)
importFrom(lubridate,as_date)
importFrom(lubridate,month)
importFrom(lubridate,ymd)
importFrom(paletteer,paletteer_dynamic)
importFrom(readr,col_character)
importFrom(readr,cols)
importFrom(readr,read_csv)
importFrom(readr,write_excel_csv)
importFrom(sf,sf_use_s2)
importFrom(stats,complete.cases)
importFrom(stats,setNames)
importFrom(stringr,str_c)
importFrom(stringr,str_replace)
importFrom(stringr,str_to_sentence)
importFrom(utils,unzip)
